@@ -1,10 +1,9 @@
 # -*- coding: utf-8 -*-
 import json
 import logging
-import traceback
-from threading import Thread
-
+import time
 import pika
+import traceback
 from pika.exceptions import StreamLostError
 from concurrent.futures import ThreadPoolExecutor
 from typing import Callable, Union
@@ -107,20 +106,28 @@ class StreamResultReceiver(object):
     3、通道按批次区分
     """
 
-    def __init__(self, processor: Union[AbstractStreamMessageProcess, Callable], rabbit_uri: str, queue_expires: int = None, max_workers: int = None):
+    def __init__(self, processor: Union[AbstractStreamMessageProcess, Callable], rabbit_uri: str, max_watcher_count: int = None, max_processor_count: int = None):
         self.processor = processor
         self.rabbit_uri = rabbit_uri
-        self.queue_expires = queue_expires or 28800
-        self.executor = ThreadPoolExecutor(max_workers=max_workers)
-        self.connection = pika.BlockingConnection(pika.URLParameters(self.rabbit_uri))
+        _max_watcher_count = max_watcher_count or 4
+        _max_processor_count = max_processor_count or 24
+        self.watcher_executor = ThreadPoolExecutor(max_workers=_max_watcher_count, thread_name_prefix='stream-watcher')
+        self.processor_executor = ThreadPoolExecutor(max_workers=_max_processor_count, thread_name_prefix='stream-processor')
         LOGGER.info(f'[流采结果]监听器初始化 rabbit_uri={self.rabbit_uri}')
 
-    def start(self, exchange_name: str, routing_name: str, queue_name: str):
-        Thread(
-            name='stream-receive',
-            target=self._receive_message,
-            kwargs={'exchange_name': exchange_name, 'routing_name': routing_name, 'queue_name': queue_name}
-        ).start()
+    def watch(self, exchange_name: str, routing_name: str, queue_name: str):
+        self.watcher_executor.submit(self._receive_message, kwargs={'exchange_name': exchange_name, 'routing_name': routing_name, 'queue_name': queue_name})
+
+    def _declare(self, exchange_name: str, routing_name: str, queue_name: str):
+        # 定义
+        connection = pika.BlockingConnection(pika.URLParameters(self.rabbit_uri))
+        channel = self.connection.channel()
+        channel.queue_declare(queue=queue_name)
+        channel.exchange_declare(exchange=exchange_name)
+        channel.queue_bind(queue=queue_name, exchange=exchange_name, routing_key=routing_name)
+        LOGGER.info(f'[流采结果]队列信息 queue={queue_name}, exchange={exchange_name}, routing={routing_name}')
+
+        return connection, channel
 
     def _receive_message(self, exchange_name: str, routing_name: str, queue_name: str):
         """
@@ -128,23 +135,18 @@ class StreamResultReceiver(object):
         """
         LOGGER.info(f'[流采结果]开始接收')
 
+        _process_func = self.processor
         if isinstance(self.processor, AbstractStreamMessageProcess):
             _process_func = self.processor.process_message
-        else:
-            _process_func = self.processor
 
-        channel = self.connection.channel()
+        connection, channel = self._declare(exchange_name, routing_name, queue_name)
         try:
-            # 定义
-            channel.queue_declare(queue=queue_name, arguments={'x-expires': self.queue_expires * 1000})
-            channel.exchange_declare(exchange=exchange_name)
-            channel.queue_bind(queue=queue_name, exchange=exchange_name, routing_key=routing_name)
-            LOGGER.info(f'[流采结果]队列信息 queue={queue_name}, exchange={exchange_name}, routing={routing_name}')
             # 接收
-            for method, properties, body in channel.consume(queue_name, auto_ack=False):
-                # 通道无活动消息一定时间后，自动终止消费（退出循环）
-                if not method and not properties:
-                    break
+            for method, properties, body in channel.consume(self.queue_name, auto_ack=False):
+                # 保持通道激活状态
+                if not method or not properties:
+                    LOGGER.error(f'[流采结果]无效消息 method={method}, properties={properties}, body={body}')
+                    continue
                 try:
                     headers = properties.headers
                     if not headers:
@@ -153,75 +155,31 @@ class StreamResultReceiver(object):
                     # 消息解析并发处理
                     msg_type = headers.get('msg_type', None)
                     body_json = json.loads(body.decode())
-                    self.executor.submit(_process_func, msg_type, body_json)
+                    self.process_executor.submit(_process_func, msg_type, body_json)
                 except (StreamLostError, ConnectionAbortedError):
                     LOGGER.error(f'[流采结果]服务端关闭链接通道')
                 except Exception:
                     LOGGER.error(f'[流采结果]解析异常 {traceback.format_exc()}')
                 # 消息确认
                 channel.basic_ack(method.delivery_tag)
+        except pika.exceptions.ConnectionClosedByBroker:
+            LOGGER.error(f'[流采结果]链接关闭异常ConnectionClosedByBroker -> 重试 {traceback.format_exc()}')
+        except pika.exceptions.AMQPChannelError as err:
+            LOGGER.error(f'[流采结果]通道关闭异常AMQPChannelError -> 重试 {traceback.format_exc()}')
+        except pika.exceptions.AMQPConnectionError:
+            LOGGER.error(f'[流采结果]链接关闭异常AMQPConnectionError -> 重试 {traceback.format_exc()}')
         except Exception:
             LOGGER.error(f'[流采结果]接收过程异常 {traceback.format_exc()}')
         finally:
             try:
                 # 关闭链接和通道（链接关闭通道自动关闭）
                 channel.close()
+                connection.close()
                 LOGGER.info(f'[流采结果]销毁队列')
             except Exception:
                 LOGGER.error(f'[流采结果]关闭链接异常 {traceback.format_exc()}')
 
-        LOGGER.info(f'[流采结果]接收完成')
-
-    def _try_reconnect(self):
-        if not self.connection or not self.connection.is_open:
-            self.connection = pika.BlockingConnection(pika.URLParameters(self.rabbit_uri))
-        channel = self.connection.channel()
-        return channel
-
-# @singleton
-# class StreamResultReceiver(object):
-#     """
-#     【单例】流式采集异步采集结果接收
-#     1、通道随批次创建，随采集完成自动销毁
-#     2、消息与队列自动创建不删除
-#     3、全局一个交换机（exchange），每个app一个路由（routing）和一个队列（queue）
-#     """
-#
-#     def __init__(self, processor: AbstractStreamMessageProcess, rabbit_uri: str,
-#                  qos: int = None, queue_expires: int = None):
-#         self.processor = processor
-#         self.rabbit_uri = rabbit_uri
-#         self.qos = qos or 10
-#         self.queue_expires = queue_expires or 28800
-#
-#     async def consume(self, exchange_name: str, routing_name: str, queue_name: str):
-#         LOGGER.info(f'[采集结果]开始接收')
-#
-#         connection = await aio_pika.connect_robust(self.rabbit_uri)
-#         channel = await connection.channel()
-#         # 服务质量保证，在非自动确认情况下，一定数目的消息没有确认，不进行消费新的消息
-#         await channel.set_qos(self.qos)
-#         queue = await channel.declare_queue(
-#             name=queue_name,
-#             arguments={'x-expires': self.queue_expires * 1000}
-#         )
-#         await channel.declare_exchange(name=exchange_name)
-#         await queue.bind(exchange=exchange_name, routing_key=routing_name)
-#         LOGGER.info(f'[采集结果]队列信息 queue={queue_name}, exchange={exchange_name}, routing={routing_name}')
-#
-#         try:
-#             async with queue.iterator() as queue_iter:
-#                 async for message in queue_iter:
-#                     self.processor.process_message(message=message)
-#                     await message.ack()
-#         except asyncio.exceptions.TimeoutError:
-#             LOGGER.info(f'[采集结果]接收超时 主动断开连接')
-#         except Exception:
-#             LOGGER.error(f'[采集结果]发生异常 {traceback.format_exc()}')
-#         finally:
-#             await queue.unbind(exchange_name, routing_name)
-#             await queue.delete()
-#             await channel.close()
-#             await connection.close()
-#
-#         LOGGER.info(f'[采集结果]接收完成')
+            # 递归重试
+            time.sleep(10)
+            LOGGER.warning(f'[流采结果]异常重试中...')
+            self._receive_message(exchange_name, routing_name, queue_name)
